package com.lz.modules.flow.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.lz.modules.flow.entity.FlowChartDetailRecord;
import com.lz.modules.flow.model.FlowChartDetailRecordDto;

import java.util.List;

/**
* <p>
* 考核模板流程记录表 服务类
* </p>
*
* @author quyixiao
* @since 2020-10-19
*/
public interface FlowChartDetailRecordService extends IService<FlowChartDetailRecord> {



	FlowChartDetailRecord selectFlowChartDetailRecordById(Long id);


	Long insertFlowChartDetailRecord(FlowChartDetailRecord flowChartDetailRecord);


	int updateFlowChartDetailRecordById(FlowChartDetailRecord flowChartDetailRecord);


	int updateCoverFlowChartDetailRecordById(FlowChartDetailRecord flowChartDetailRecord);


	int deleteFlowChartDetailRecordById(Long id);


	List<FlowChartDetailRecord> selectFlowChartDetailRecordByGroupIdAndChartId(Long groupId, Long chartId);

    List<FlowChartDetailRecord> selectFlowChartDetailRecordByGroupId(Long groupId);

    int insertFlowChartDetailRecords(List<FlowChartDetailRecord> inserts);

	int updateCoverFlowChartDetailRecordByIds(List<FlowChartDetailRecord> updaes);

    List<FlowChartDetailRecord> selectFlowChartDetailRecordsByFlowProcess(Long groupId, int flowProcess);

    //获取开启节点的
	List<FlowChartDetailRecord> selectOpenFlowChartDetailRecordByGroupIdAndChartId(Long groupId, Long chartId);
}